@@ -1,13 +1,17 @@
+from datetime import timedelta
+from itertools import chain
 from typing import Optional
 
 import click
 from flask import current_app as app
 from flask.cli import with_appcontext
+from timely_beliefs.beliefs.queries import query_unchanged_beliefs
 
 from flexmeasures.data import db
 from flexmeasures.data.models.user import Account, AccountRole, RolesAccounts, User
 from flexmeasures.data.models.generic_assets import GenericAsset
-from flexmeasures.data.scripts.data_gen import get_affected_classes
+from flexmeasures.data.models.time_series import Sensor, TimedBelief
+from flexmeasures.data.schemas.generic_assets import GenericAssetIdField
 from flexmeasures.data.services.users import find_user_by_email, delete_user
 
 
@@ -66,10 +70,9 @@ def delete_account(id: int, force: bool):
                 + ",".join([ga.name for ga in generic_assets])
                 + "\n"
             )
-        if not click.confirm(prompt):
-            raise click.Abort()
+        click.confirm(prompt, abort=True)
     for user in account.users:
-        print(f"Deleting user {user} (and assets & data) ...")
+        print(f"Deleting user {user} ...")
         delete_user(user)
     for role_account_association in RolesAccounts.query.filter_by(
         account_id=account.id
@@ -94,41 +97,36 @@ def delete_account(id: int, force: bool):
 @click.option(
     "--force/--no-force", default=False, help="Skip warning about consequences."
 )
-def delete_user_and_data(email: str, force: bool):
+def delete_a_user(email: str, force: bool):
     """
     Delete a user & also their assets and data.
     """
     if not force:
-        # TODO: later, when assets belong to accounts, remove this.
-        prompt = f"Delete user '{email}', including all their assets and data?"
-        if not click.confirm(prompt):
-            raise click.Abort()
+        prompt = f"Delete user '{email}'?"
+        click.confirm(prompt, abort=True)
     the_user = find_user_by_email(email)
     if the_user is None:
         print(f"Could not find user with email address '{email}' ...")
         return
     delete_user(the_user)
-    app.db.session.commit()
+    db.session.commit()
 
 
-def confirm_deletion(
-    structure: bool = False,
-    data: bool = False,
-    is_by_id: bool = False,
-):
-    affected_classes = get_affected_classes(structure, data)
-    prompt = "This deletes all %s entries from %s.\nDo you want to continue?" % (
-        " and ".join(
-            ", ".join(
-                [affected_class.__tablename__ for affected_class in affected_classes]
-            ).rsplit(", ", 1)
-        ),
-        app.db.engine,
-    )
-    if is_by_id:
-        prompt = prompt.replace(" all ", " ")
-    if not click.confirm(prompt):
-        raise click.Abort()
+@fm_delete_data.command("asset")
+@with_appcontext
+@click.option("--id", "asset", type=GenericAssetIdField())
+@click.option(
+    "--force/--no-force", default=False, help="Skip warning about consequences."
+)
+def delete_asset_and_data(asset: GenericAsset, force: bool):
+    """
+    Delete an asset & also its sensors and data.
+    """
+    if not force:
+        prompt = f"Delete {asset}, including all its sensors and data?"
+        click.confirm(prompt, abort=True)
+    db.session.delete(asset)
+    db.session.commit()
 
 
 @fm_delete_data.command("structure")
@@ -139,17 +137,16 @@ def confirm_deletion(
 def delete_structure(force):
     """
     Delete all structural (non time-series) data like assets (types),
-    markets (types) and weather sensors (types) and users.
-
-    TODO: This could in our future data model (currently in development) be replaced by
-    `flexmeasures delete generic-asset-type`, `flexmeasures delete generic-asset`
-    and `flexmeasures delete sensor`.
+    sources, roles and users.
     """
     if not force:
-        confirm_deletion(structure=True)
+        click.confirm(
+            f"Sure to delete all asset(type)s, sources, roles and users from {db.engine}?",
+            abort=True,
+        )
     from flexmeasures.data.scripts.data_gen import depopulate_structure
 
-    depopulate_structure(app.db)
+    depopulate_structure(db)
 
 
 @fm_delete_data.command("measurements")
@@ -168,10 +165,10 @@ def delete_measurements(
 ):
     """Delete measurements (ex-post beliefs, i.e. with belief_horizon <= 0)."""
     if not force:
-        confirm_deletion(data=True, is_by_id=sensor_id is not None)
+        click.confirm(f"Sure to delete all measurements from {db.engine}?", abort=True)
     from flexmeasures.data.scripts.data_gen import depopulate_measurements
 
-    depopulate_measurements(app.db, sensor_id)
+    depopulate_measurements(db, sensor_id)
 
 
 @fm_delete_data.command("prognoses")
@@ -190,10 +187,130 @@ def delete_prognoses(
 ):
     """Delete forecasts and schedules (ex-ante beliefs, i.e. with belief_horizon > 0)."""
     if not force:
-        confirm_deletion(data=True, is_by_id=sensor_id is not None)
+        click.confirm(f"Sure to delete all prognoses from {db.engine}?", abort=True)
     from flexmeasures.data.scripts.data_gen import depopulate_prognoses
 
-    depopulate_prognoses(app.db, sensor_id)
+    depopulate_prognoses(db, sensor_id)
+
+
+@fm_delete_data.command("unchanged-beliefs")
+@with_appcontext
+@click.option(
+    "--sensor-id",
+    type=int,
+    help="Delete unchanged (time series) data for a single sensor only. Follow up with the sensor's ID. ",
+)
+@click.option(
+    "--delete-forecasts/--keep-forecasts",
+    "delete_unchanged_forecasts",
+    default=True,
+    help="Use the --keep-forecasts flag to keep unchanged beliefs with a positive belief horizon (forecasts).",
+)
+@click.option(
+    "--delete-measurements/--keep-measurements",
+    "delete_unchanged_measurements",
+    default=True,
+    help="Use the --keep-measurements flag to keep beliefs with a zero or negative belief horizon (measurements, nowcasts and backcasts).",
+)
+def delete_unchanged_beliefs(
+    sensor_id: Optional[int] = None,
+    delete_unchanged_forecasts: bool = True,
+    delete_unchanged_measurements: bool = True,
+):
+    """Delete unchanged beliefs (i.e. updated beliefs with a later belief time, but with the same event value)."""
+    q = db.session.query(TimedBelief)
+    if sensor_id:
+        sensor = Sensor.query.filter(Sensor.id == sensor_id).one_or_none()
+        if sensor is None:
+            print(f"Failed to delete any beliefs: no sensor found with id {sensor_id}.")
+            return
+        q = q.filter(TimedBelief.sensor_id == sensor.id)
+    num_beliefs_before = q.count()
+
+    unchanged_queries = []
+    num_forecasts_up_for_deletion = 0
+    num_measurements_up_for_deletion = 0
+    if delete_unchanged_forecasts:
+        q_unchanged_forecasts = query_unchanged_beliefs(
+            db.session,
+            TimedBelief,
+            q.filter(
+                TimedBelief.belief_horizon > timedelta(0),
+            ),
+            include_non_positive_horizons=False,
+        )
+        unchanged_queries.append(q_unchanged_forecasts)
+        num_forecasts_up_for_deletion = q_unchanged_forecasts.count()
+    if delete_unchanged_measurements:
+        q_unchanged_measurements = query_unchanged_beliefs(
+            db.session,
+            TimedBelief,
+            q.filter(
+                TimedBelief.belief_horizon <= timedelta(0),
+            ),
+            include_positive_horizons=False,
+        )
+        unchanged_queries.append(q_unchanged_measurements)
+        num_measurements_up_for_deletion = q_unchanged_measurements.count()
+
+    num_beliefs_up_for_deletion = (
+        num_forecasts_up_for_deletion + num_measurements_up_for_deletion
+    )
+    prompt = f"Delete {num_beliefs_up_for_deletion} unchanged beliefs ({num_measurements_up_for_deletion} measurements and {num_forecasts_up_for_deletion} forecasts) out of {num_beliefs_before} beliefs?"
+    click.confirm(prompt, abort=True)
+
+    beliefs_up_for_deletion = list(chain(*[q.all() for q in unchanged_queries]))
+    batch_size = 10000
+    for i, b in enumerate(beliefs_up_for_deletion, start=1):
+        if i % batch_size == 0 or i == num_beliefs_up_for_deletion:
+            print(f"{i} beliefs processed ...")
+        db.session.delete(b)
+    print(f"Removing {num_beliefs_up_for_deletion} beliefs ...")
+    db.session.commit()
+    num_beliefs_after = q.count()
+    print(f"Done! {num_beliefs_after} beliefs left")
+
+
+@fm_delete_data.command("nan-beliefs")
+@with_appcontext
+@click.option(
+    "--sensor-id",
+    type=int,
+    help="Delete NaN time series data for a single sensor only. Follow up with the sensor's ID.",
+)
+def delete_nan_beliefs(sensor_id: Optional[int] = None):
+    """Delete NaN beliefs."""
+    q = db.session.query(TimedBelief)
+    if sensor_id is not None:
+        q = q.filter(TimedBelief.sensor_id == sensor_id)
+    query = q.filter(TimedBelief.event_value == float("NaN"))
+    prompt = f"Delete {query.count()} NaN beliefs out of {q.count()} beliefs?"
+    click.confirm(prompt, abort=True)
+    query.delete()
+    db.session.commit()
+    print(f"Done! {q.count()} beliefs left")
+
+
+@fm_delete_data.command("sensor")
+@with_appcontext
+@click.option(
+    "--id",
+    "sensor_id",
+    type=int,
+    required=True,
+    help="Delete a single sensor and its (time series) data. Follow up with the sensor's ID.",
+)
+def delete_sensor(
+    sensor_id: int,
+):
+    """Delete a sensor and all beliefs about it."""
+    sensor = Sensor.query.get(sensor_id)
+    n = TimedBelief.query.filter(TimedBelief.sensor_id == sensor_id).delete()
+    db.session.delete(sensor)
+    click.confirm(
+        f"Really delete sensor {sensor_id}, along with {n} beliefs?", abort=True
+    )
+    db.session.commit()
 
 
 app.cli.add_command(fm_delete_data)
